kernelopts(assertlevel=2): # be strict on all assertions while testing
kernelopts(opaquemodules=false): # allow testing of internal routines
read "NewSLO.mpl":
if not (NewSLO :: `module`) then
  WARNING("loading NewSLO failed");
  `quit`(3);
end if;

with(NewSLO):


Ite := proc(cond,thn,othw)
    Msum(Weight(piecewise(cond,1,0),thn),Weight(piecewise(cond,0,1),othw))
end proc;

myOr := proc(a,b)
   Ite(a=true,Ret(true),Ret(b))
end proc;

myAnd := proc(a,b)
   Ite(a=true,Ret(b),Ret(false))
end proc;

Bernoulli := proc(p)
    Msum(Weight(p,Ret(true)),Weight(1-p,Ret(false)))
end proc;

Observe := proc(cond,rest)
    Weight(piecewise(cond,1,0),rest)
end proc;

RunNotNormalized := proc(m)
    m := fromLO(improve(toLO(m)));
end proc;

Normalize := proc(m)
    Weight(1/integrate(m,x->1),m)
end proc;

Run := proc(m)
    simplify(fromLO(improve(toLO(Normalize(m)))));
end proc;

burglarAlarm :=
             Bind(Bernoulli(1/10000),earthquake,
             Bind(Bernoulli(1/1000),burglary,
             Bind(myOr(earthquake,burglary),alarm,
             Bind(Ite(earthquake=true,Bernoulli(7/10),Bernoulli(99/100)),phoneWorking,
             Bind(Ite(alarm=true,Ite(earthquake=true,Bernoulli(8/10),Bernoulli(6/10)),Bernoulli(2/10)),maryWakes,
             Bind(myAnd(maryWakes,phoneWorking),called,
             Observe(called=true,Ret(burglary))))))));
             # Ret(Pair(called,burglary))))))));
             # Ret(burglary)))))));

Run(burglarAlarm);

# ClinicalTrial1 is in separate file ClinicalTrial1.mpl

coinBias :=
         Bind(BetaD(2,5),bias,
         Bind(Bernoulli(bias),tossResult1,
         Bind(Bernoulli(bias),tossResult2,
         Bind(Bernoulli(bias),tossResult3,
         Bind(Bernoulli(bias),tossResult4,
         Bind(Bernoulli(bias),tossResult5,
         Observe(tossResult1=true,
         Observe(tossResult2=true,
         Observe(tossResult3=false,
         Observe(tossResult4=true,
         Observe(tossResult5=false,
         Ret(bias))))))))))));

RunNotNormalized(coinBias);

# DigitRecognition is in separate file DigitRecognition.mpl

grass :=
      Bind(Bernoulli(1/2),cloudy,
      Bind(Ite(cloudy=true,Bernoulli(8/10),Bernoulli(2/10)),rain,
      Bind(Ite(cloudy=true,Bernoulli(1/10),Bernoulli(1/2)),sprinkler,
      Bind(Bernoulli(7/10),temp1,
      Bind(myAnd(temp1,rain),wetRoof,
      Bind(Bernoulli(9/10),temp2,
      Bind(Bernoulli(9/10),temp3,
      Bind(myAnd(temp2,rain),temp2AndRain,
      Bind(myAnd(temp3,sprinkler),temp3AndSprinkler,
      Bind(myOr(temp2AndRain,temp3AndSprinkler),wetGrass,
      Observe(wetGrass=true,
      Ret(rain))))))))))));

Run(grass);

# HIV and LinearRegression1 are not supported

noisyOrModel :=
             Bind(Bernoulli(1/2),n0,
             Bind(Bernoulli(1/2),n4,
             Bind(Ite(n0=true,Bernoulli(4/5),Bernoulli(1/10)),n1,
             Bind(Ite(n0=true,Bernoulli(4/5),Bernoulli(1/10)),n21,
             Bind(Ite(n4=true,Bernoulli(4/5),Bernoulli(1/10)),n22,
             Bind(Ite(n4=true,Bernoulli(4/5),Bernoulli(1/10)),n33,
             Bind(myOr(n21,n22),n2,
             Bind(Ite(n1=true,Bernoulli(4/5),Bernoulli(1/10)),n31,
             Bind(Ite(n2=true,Bernoulli(4/5),Bernoulli(1/10)),n32,
             Bind(myOr(n31,n32),n31Orn32,
             Bind(myOr(n31Orn32,n33),n3,
             Ret(n3))))))))))));

Run(noisyOrModel);


surveyResultUnbias :=
                   Bind(BetaD(1,1),bias1,
                   Bind(BetaD(1,1),bias2,
                   Bind(Ret(3141*bias1),mean1,
                   Bind(Ret(2718*bias2),mean2,
                   Bind(Ret(mean1-mean1*bias1),variance1,
                   Bind(Ret(mean2-mean2*bias2),variance2,
                   Bind(Gaussian(mean1,variance1^(1/2)),votes1,
                   Bind(Gaussian(mean2,variance2^(1/2)),votes2,
                   Bind(Ret(bias1),ansBias1,
                   Bind(Ret(bias2),ansBias2,
                   Bind(Ret(bias1),ansBias3,
                   Bind(Ret(bias1),ansBias4,
                   Bind(Ret(bias2),ansBias5,
                   Bind(Bernoulli(ansBias1),answer1,
                   Bind(Bernoulli(ansBias2),answer2,
                   Bind(Bernoulli(ansBias3),answer3,
                   Bind(Bernoulli(ansBias4),answer4,
                   Bind(Bernoulli(ansBias5),answer5,
                   Observe(answer1=true,
                   Observe(answer2=false,
                   Observe(answer3=true,
                   Observe(answer4=true,
                   Observe(answer5=true,
                   Ret(Pair(Pair(Pair(bias1,bias2),Pair(Pair(mean1,mean2),Pair(variance1,variance2)),Pair(votes1,votes2)))))))))))))))))))))))))));
                   #Ret(bias1))))))))))))))))))))))));

RunNotNormalized(surveyResultUnbias);

trueSkill :=
	  Bind(Gaussian(100,10^(1/2)),skillA,
	  Bind(Gaussian(100,10^(1/2)),skillB,
	  Bind(Gaussian(100,10^(1/2)),skillC,
	  Bind(Gaussian(skillA,15),perfA1,
	  Bind(Gaussian(skillB,15),perfB1,
	  Observe(perfA1 > perfB1,
	  Bind(Gaussian(skillB,15),perfB2,
	  Bind(Gaussian(skillC,15),perfC2,
	  Observe(perfB2 > perfC2,
	  Bind(Gaussian(skillA,15),perfA3,
	  Bind(Gaussian(skillC,15),perfC3,
	  Observe(perfA3 > perfC3,
	  Ret(Pair(Pair(skillA,skillB),skillC))))))))))))));

RunNotNormalized(trueSkill);


twoCoins :=
         Bind(Bernoulli(1/2),firstCoin,
         Bind(Bernoulli(1/2),secondCoin,
         Bind(myAnd(firstCoin,secondCoin),bothHeads,
         Observe(bothHeads=false,
         Ret(Pair(firstCoin,secondCoin))))));

Run(twoCoins); # Cannot compute expectation


gaussMax :=
         Bind(Gaussian(0,1),x,
         Bind(Gaussian(0,1),y,
         Bind(Ret(max(x,y)),z,
         Ret(z))));

Run(gaussMax);

gaussSum :=
         Bind(Gaussian(0,1),x1,
         Bind(Gaussian(0,1),x2,
         Bind(Gaussian(0,1),x3,
         Bind(Ret(true),ind1,
         Bind(Ret(false),ind2,
         Bind(Ret(true),ind3,
         Bind(Ret(0),r1,
         Bind(Ite(ind1=true,Ret(r1+x1),Ret(r1)),r2,
         Bind(Ite(ind2=true,Ret(r2+x2),Ret(r2)),r3,
         Bind(Ite(ind3=true,Ret(r3+x3),Ret(r3)),r4,
         Ret(r4)))))))))));

Run(gaussSum);


bayesPointMachine :=
		  Bind(Ret(1/10),noise,
		  Bind(Gaussian(0,1),weights1,
		  Bind(Gaussian(0,1),weights2,
		  Bind(Gaussian(0,1),weights3,

		  Bind(Ret(63*weights1 + 38*weights2 + weights3),mean1,
		  Bind(Gaussian(mean1,noise^(1/2)),tmp1,
		  Bind(Ite(tmp1 > 0,Ret(true),Ret(false)),result1,
		  Observe(result1 = true,

		  Bind(Ret(16*weights1 + 23*weights2 + weights3),mean2,
 		  Bind(Gaussian(mean2,noise^(1/2)),tmp2,
		  Bind(Ite(tmp2 > 0,Ret(true),Ret(false)),result2,
		  Observe(result2 = false,

		  Bind(Ret(28*weights1 + 40*weights2 + weights3),mean3,
 		  Bind(Gaussian(mean3,noise^(1/2)),tmp3,
		  Bind(Ite(tmp3 > 0,Ret(true),Ret(false)),result3,
		  Observe(result3 = true,

		  Bind(Ret(55*weights1 + 27*weights2 + weights3),mean4,
 		  Bind(Gaussian(mean4,noise^(1/2)),tmp4,
		  Bind(Ite(tmp4 > 0,Ret(true),Ret(false)),result4,
		  Observe(result4 = true,

		  Bind(Ret(22*weights1 + 18*weights2 + weights3),mean5,
 		  Bind(Gaussian(mean5,noise^(1/2)),tmp5,
		  Bind(Ite(tmp5 > 0,Ret(true),Ret(false)),result5,
		  Observe(result5 = false,

		  Bind(Ret(20*weights1 + 40*weights2 + weights3),mean6,
 		  Bind(Gaussian(mean6,noise^(1/2)),tmp6,
		  Bind(Ite(tmp6 > 0,Ret(true),Ret(false)),result6,
		  Observe(result6 = false,
		  
		  Ret(Pair(Pair(weights1,weights2),weights,3))))))))))))))))))))))))))))));


RunNotNormalized(bayesPointMachine);

clickGraph :=
           Bind(BetaD(1,1),similarityAll,

           Bind(Bernoulli(similarityAll),sim_i1,
           Bind(BetaD(1,1),beta1_i1,
           Bind(Ite(sim_i1=true,Ret(beta1_i1),BetaD(1,1)),beta2_i1,
           Bind(Bernoulli(beta1_i1),sim_01,
           Bind(Bernoulli(beta2_i1),sim_11,
           
           Observe(sim_01=true,Observe(sim_11=true,

           Bind(Bernoulli(similarityAll),sim_i2,
           Bind(BetaD(1,1),beta1_i2,
           Bind(Ite(sim_i2=true,Ret(beta1_i2),BetaD(1,1)),beta2_i2,
           Bind(Bernoulli(beta1_i2),sim_02,
           Bind(Bernoulli(beta2_i2),sim_12,

           Observe(sim_02=true,Observe(sim_12=true,

           Bind(Bernoulli(similarityAll),sim_i3,
           Bind(BetaD(1,1),beta1_i3,
           Bind(Ite(sim_i3=true,Ret(beta1_i3),BetaD(1,1)),beta2_i3,
           Bind(Bernoulli(beta1_i3),sim_03,
           Bind(Bernoulli(beta2_i3),sim_13,

           Observe(sim_03=true,Observe(sim_13=true,

           Bind(Bernoulli(similarityAll),sim_i4,
           Bind(BetaD(1,1),beta1_i4,
           Bind(Ite(sim_i4=true,Ret(beta1_i4),BetaD(1,1)),beta2_i4,
           Bind(Bernoulli(beta1_i4),sim_04,
           Bind(Bernoulli(beta2_i4),sim_14,

           Observe(sim_04=false,Observe(sim_14=false,

           Bind(Bernoulli(similarityAll),sim_i5,
           Bind(BetaD(1,1),beta1_i5,
           Bind(Ite(sim_i5=true,Ret(beta1_i5),BetaD(1,5)),beta2_i5,
           Bind(Bernoulli(beta1_i5),sim_05,
           Bind(Bernoulli(beta2_i5),sim_15,

           Observe(sim_05=false,Observe(sim_15=false,

           Ret(similarityAll)))))))))))))))))))))))))))))))))))));

RunNotNormalized(clickGraph);


clinicalTrial :=
              Bind(Ret(false),controlGroup1,
              Bind(Ret(false),controlGroup2,
              Bind(Ret(true),controlGroup3,
              Bind(Ret(false),controlGroup4,
              Bind(Ret(false),controlGroup5,

              Bind(Ret(true),treatedGroup1,
              Bind(Ret(false),treatedGroup2,
              Bind(Ret(true),treatedGroup3,
              Bind(Ret(true),treatedGroup4,
              Bind(Ret(true),treatedGroup5,

              Bind(Bernoulli(1/2),isEffective,
              Bind(BetaD(1,1),probIfTreated,
              Bind(Ite(isEffective=true,BetaD(1,1),Ret(probIfTreated)),probIfControl,
              # Bind(Ret(fromMarginal(isEffective),isEffectiveP, # cannot do this

              Bind(Bernoulli(probIfControl),r1,
              Observe(r1=controlGroup1,
              Bind(Bernoulli(probIfControl),r2,
              Observe(r2=controlGroup2,
              Bind(Bernoulli(probIfControl),r3,
              Observe(r3=controlGroup3,
              Bind(Bernoulli(probIfControl),r4,
              Observe(r4=controlGroup4,
              Bind(Bernoulli(probIfControl),r5,
              Observe(r5=controlGroup5,

              Bind(Bernoulli(probIfTreated),r1,
              Observe(r1=treatedGroup1,
              Bind(Bernoulli(probIfTreated),r2,
              Observe(r2=treatedGroup2,
              Bind(Bernoulli(probIfTreated),r3,
              Observe(r3=treatedGroup3,
              Bind(Bernoulli(probIfTreated),r4,
              Observe(r4=treatedGroup4,
              Bind(Bernoulli(probIfTreated),r5,
              Observe(r5=treatedGroup5,

              Observe(isEffective=true,
              Ret(Pair(probIfTreated,probIfControl))))))))))))))))))))))))))))))))))));       

RunNotNormalized(clinicalTrial);



coins :=
         Bind(Bernoulli(1/2),firstCoin,
         Bind(Bernoulli(1/2),secondCoin,
         Bind(myAnd(firstCoin,secondCoin),bothHeads,
         Observe(bothHeads=false,
         Ret(Pair(firstCoin,secondCoin))))));

Run(coins);

evidenceModel1 :=
               Bind(Bernoulli(1/2),evidence,
               Bind(Ite(evidence=true,
                        Bind(Bernoulli(1/2),coin,
                        Observe(coin=true,
                        Ret(Unit))),
                        Ret(Unit)),unit,
               Ret(evidence)));

Run(evidenceModel1);


evidenceModel2 :=
               Bind(Bernoulli(1/2),evidence,
               Bind(Ite(evidence=true,
                        Bind(Bernoulli(1/2),coin1,
                        Observe(coin1=true,
                        Ret(coin1))),
                        Bernoulli(1/2)),coin,
               Ret(coin)));

Run(evidenceModel2);

# LearningGaussian is not supported

murderMystery := 
	      Bind(Ret(true),gunFound,
	      Bind(Bernoulli(3/10),aliceDunnit,
	      Bind(Ite(aliceDunnit=true,Bernoulli(3/100),Bernoulli(8/10)),withGun,
	      Observe(withGun=gunFound,
	      Bind(Ret(aliceDunnit),posterior,
	      Ret(posterior))))));

Run(murderMystery);
